<?php
defined('_JEXEC') or die;

require_once PERFECTDASHBOARD_J_PLUGIN_HELPER_PATH . 'Joomla.php';

class PerfectDashboard_Cms_Joomla_Config extends PerfectDashboard_Config
{
	/**
	 * @var JRegistry|Joomla\Registry\Registry
	 */
	private $params;

	/**
	 * @return string
	 */
	protected function getOptionSiteUrl()
	{
		return JUri::root(false);
	}

	/**
	 * @return string
	 */
	protected function getOptionSiteBackendUrl()
	{
		if (PerfectDashboard_Cms_Joomla_Helper_Joomla::isAdmin())
		{
			return JUri::base(false);
		}

		return rtrim(JUri::root(false), '/') . '/' . basename(JPATH_ADMINISTRATOR);
	}


	/**
	 * @param string     $key
	 * @param null|mixed $default
	 *
	 * @return mixed
	 */
	protected function getOption($key, $default = null)
	{
		$value = $this->getAllOptions()->get($key, $default);
		if (($key == 'read_token' || $key == 'write_token') && is_null($value) &&
			version_compare(JVERSION, '3.0', '>='))
		{
			// Get legacy token to finish update process
			jimport('joomla.component.helper');

			return JComponentHelper::getParams('com_perfectdashboard')->get('key', $default);
		}

		return $value;
	}

	/**
	 * @param string $key
	 * @param mixed  $value
	 *
	 * @return bool
	 */
	protected function setOption($key, $value)
	{
		$this->getAllOptions()->set($key, $value);

		return $this->save();
	}

	/**
	 * @param string $key
	 *
	 * @return bool
	 */
	protected function removeOption($key)
	{
		$options = $this->getAllOptions()->toArray();

		if (isset($options[$key]))
		{
			unset($options[$key]);
		}

		$this->params = PerfectDashboard_Cms_Joomla_Helper_Joomla::getRegistry($options);

		return $this->save();
	}

	/**
	 * @return bool
	 */
	protected function removeAllOptions()
	{
		$this->params = PerfectDashboard_Cms_Joomla_Helper_Joomla::getRegistry();

		return $this->save();
	}

	/**
	 * @return JRegistry|Joomla\Registry\Registry
	 */
	private function getAllOptions()
	{
		if (empty($this->params))
		{
			$this->params = PerfectDashboard_Cms_Joomla_Helper_Joomla::getRegistry();

			$db    = JFactory::getDbo();
			$query = $db->getQuery(true)
				->select($db->qn('params'))
				->from($db->qn('#__extensions'))
				->where(PerfectDashboard_Cms_Joomla_Helper_Joomla::getChildWhereCondition());

			try
			{
				$this->params->loadString(
					$db->setQuery($query)->loadResult()
				);
			}
			catch (Exception $e)
			{

			}
		}

		return $this->params;
	}

	/**
	 * @return bool
	 */
	private function save()
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true)
			->update($db->qn('#__extensions'))
			->set($db->qn('params') . ' = ' . $db->q($this->getAllOptions()->toString()))
			->where(PerfectDashboard_Cms_Joomla_Helper_Joomla::getChildWhereCondition());

		try
		{
			return $db->setQuery($query)->execute();
		}
		catch (Exception $e)
		{

		}

		return false;
	}
}