<?php
defined('_JEXEC') or die;

class PerfectDashboard_Cms_Joomla_Helper_Whitelabeller
{
	private $name = null;
	private $child_page = null;
	private $login_page = null;
	private $hide_child = null;

	/**
	 * PerfectDashboard_Cms_Joomla_Helper_Whitelabeller constructor.
	 *
	 * @param string $name
	 * @param string $child_page
	 * @param string $login_page
	 * @param bool   $hide_child
	 */
	public function __construct($name, $child_page, $login_page, $hide_child)
	{
		$this->name       = (string) $name;
		$this->child_page = (string) $child_page;
		$this->login_page = (string) $login_page;
		$this->hide_child = (bool) $hide_child;
	}

	public function handle()
	{
		$this->setName();
		$this->setChildPage();
		$this->setLoginPage();
		$this->hideChild();
	}

	/**
	 * Override name of Perfect Dashboard extension.
	 *
	 * @throws PerfectDashboard_Exception_Response
	 *
	 * @return bool
	 */
	public function setName()
	{
		PerfectDashboard_Config::set('whitelabel_name', $this->name);

		// Get all available languages on site.
		foreach (JLanguage::getKnownLanguages() as $lang)
		{
			// Set language for right JText value.
			$jlang = JFactory::getLanguage();
			$jlang->setDefault($lang['tag']);
			$jlang->load();
			$jlang->load('plg_system_perfectdashboard');

			$override_filename = JPATH_ADMINISTRATOR . '/language/overrides/' . $lang['tag'] . '.override.ini';

			$override_name = array(
				'language' => $lang['name'] . ' [' . $lang['tag'] . ']',
				'client'   => 'Administrator',
				'file'     => $override_filename
			);

			$name_overrides = array(
				'PERFECT DASHBOARD'           => $this->name,
				'PLG_SYSTEM_PERFECTDASHBOARD' => $this->name,
			);

			if (empty($this->name))
			{
				foreach ($name_overrides as $id => $item)
				{
					$override_name['id'] = $id;

					if (!$this->deleteOverride($override_name))
					{
						throw new PerfectDashboard_Exception_Response('Could not delete overrides.');
					}
				}
			}
			else
			{
				foreach ($name_overrides as $id => $item)
				{
					$override_name['key']      = $id;
					$override_name['override'] = $item;
					$override_name['id']       = $id;

					if (!$this->saveOverride($override_name))
					{
						throw new PerfectDashboard_Exception_Response('Could not save overrides.');
					}
				}
			}
		}

		return true;
	}

	/**
	 *
	 * @return bool
	 */
	public function setChildPage()
	{
		if (empty($this->child_page))
		{
			return PerfectDashboard_Config::remove('whitelabel_child_page');
		}
		else
		{
			return PerfectDashboard_Config::set('whitelabel_child_page', $this->child_page);
		}
	}

	/**
	 * Override content of site's login page.
	 *
	 * @return bool
	 */
	public function setLoginPage()
	{
		// Check if custom module in position "login" exist - we store login_page there.
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
			->select($db->qn('id'))
			->from($db->qn('#__modules'))
			->where(array(
				$db->qn('title') . '=' . $db->q('Admin login page'),
				$db->qn('position') . '=' . $db->q('login'),
				$db->qn('module') . '=' . $db->q('mod_custom'),
			));

		try
		{
			$module_id = $db->setQuery($query)
				->loadResult();
		}
		catch (Exception $e)
		{
		}

		try
		{
			if (empty($module_id))
			{
				// Module doesn't exist - add it.
				$module            = new stdClass();
				$module->title     = 'Admin login page';
				$module->content   = $this->login_page;
				$module->ordering  = 1;
				$module->position  = 'login';
				$module->published = 1;
				$module->module    = 'mod_custom';
				$module->access    = 1;
				$module->params    = '{"prepare_content":"1"}';
				$module->client_id = 1;
				$module->language  = '*';

				$db->insertObject('#__modules', $module, 'id');

				if (!empty($module->id))
				{
					$modules_menu           = new stdClass();
					$modules_menu->moduleid = $module->id;
					$modules_menu->menuid   = 0;

					$db->insertObject('#__modules_menu', $modules_menu);
				}
			}
			else
			{
				$query = $db->getQuery(true)
					->update($db->qn('#__modules'))
					->set($db->qn('content') . '=' . $db->q($this->login_page))
					->where($db->qn('id') . '=' . (int) $module_id);

				$db->setQuery($query)
					->execute();
			}
		}
		catch (Exception $e)
		{
			return false;
		}

		return true;
	}

	/**
	 * Protect extension.
	 *
	 * @return bool
	 */
	public function hideChild()
	{
		PerfectDashboard_Config::set('hide_child', $this->hide_child);

		$db    = JFactory::getDbo();
		$query = $db->getQuery(true)
			->update($db->qn('#__extensions'))
			->set($db->qn('protected') . ' = ' . $db->q((int) $this->hide_child, false))
			->where(PerfectDashboard_Cms_Joomla_Helper_Joomla::getChildWhereCondition());

		try
		{
			return $db->setQuery($query)->execute();
		}
		catch (Exception $e)
		{

		}

		return false;
	}

	/**
	 * Override language constants values.
	 * It's part of administrator\components\com_languages\models\override.php save method, not used directly because off JPATH_COMPONENT usage there.
	 *
	 * @param array $data
	 *
	 * @return bool
	 */
	private function saveOverride($data)
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_languages/helpers/languages.php';

		// Parse the override.ini file in oder to get the keys and strings.
		$strings = LanguagesHelper::parseFile($data['file']);

		if (isset($strings[$data['id']]))
		{
			// If an existent string was edited check whether
			// the name of the constant is still the same.
			if ($data['key'] == $data['id'])
			{
				// If yes, simply override it.
				$strings[$data['key']] = $data['override'];
			}
			else
			{
				// If no, delete the old string and prepend the new one.
				unset($strings[$data['id']]);
				$strings = array($data['key'] => $data['override']) + $strings;
			}
		}
		else
		{
			// If it is a new override simply prepend it.
			$strings = array($data['key'] => $data['override']) + $strings;
		}

		return $this->writeOverride($strings, $data);
	}

	/**
	 * Delete override of language constants values.
	 * It's part of administrator\components\com_languages\models\overrides.php delete method, not used directly because of JPATH_COMPONENT usage there.
	 *
	 * @param array $data
	 *
	 * @return bool
	 */
	private function deleteOverride($data)
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_languages/helpers/languages.php';

		// Parse the override.ini file in oder to get the keys and strings.
		$strings = LanguagesHelper::parseFile($data['file']);

		// Unset strings that shall be deleted
		if (isset($strings[$data['id']]))
		{
			unset($strings[$data['id']]);
		}

		foreach ($strings as $key => $string)
		{
			$strings[$key] = str_replace('"', '"_QQ_"', $string);
		}

		return $this->writeOverride($strings, $data);
	}

	/**
	 * Helper method used in saveOverride and deleteOverride.
	 *
	 * @param array $strings
	 * @param array $data
	 *
	 * @return bool
	 */
	private function writeOverride($strings, $data)
	{
		foreach ($strings as $key => $string)
		{
			$strings[$key] = str_replace('"', '"_QQ_"', $string);
		}

		// Write override.ini file with the strings.
		$registry = PerfectDashboard_Cms_Joomla_Helper_Joomla::getRegistry();
		$registry->loadObject($strings);
		$reg = $registry->toString('INI');

		$file_manager = PerfectDashboard_Filemanager::getInstance();

		if ($file_manager->put_contents($data['file'], $reg))
		{
			return true;
		}
	}
}