<?php
defined('PERFECTDASHBOARD_LIB') or die;

class PerfectDashboard_Task_PostBackuptoolInstall extends PerfectDashboard_Task_Base
{
	/**
	 * @throws PerfectDashboard_Exception_Response
	 *
	 * @return array
	 */
	public function doTask()
	{
		if (PERFECTDASHBOARD_STAGE != 'app')
		{
			// Get credentials from the request payload
			$dir      = $this->input('directory');
			$login    = $this->input('login');
			$password = $this->input('password');
			$secret   = $this->input('secret');
		}

		if (empty($dir) || empty($login) || empty($password) || empty($secret))
		{
			// Get credentials from Perfect Dashboard
			$query = array(
				'pd_site_id' => (int) $this->input('site_id'),
			);

			$query['pd_signature'] = PerfectDashboard_Authentication::getInstance()
				->getSignature($query);
			if (!$query['pd_site_id'] || !$query['pd_signature'])
			{
				throw new PerfectDashboard_Exception_Response('Missing required parameters', 400);
			}

			$response = PerfectDashboard_Request::get(
				PerfectDashboard_Config::getPdUrl() . 'api/1.0/child/backuptool/credentials',
				$query,
				array(
					'Content-Type' => 'application/json',
				)
			);

			$dir      = !empty($response->body->directory) ? $response->body->directory : null;
			$login    = !empty($response->body->login) ? $response->body->login : null;
			$password = !empty($response->body->password) ? $response->body->password : null;
			$secret   = !empty($response->body->secret) ? $response->body->secret : null;
		}

		if (empty($dir) || empty($login) || empty($password) || empty($secret))
		{
			throw new PerfectDashboard_Exception_Response('Failed to get backup tool credentials', 400);
		}

		return PerfectDashboard_Backuptool::getInstance()
			->install($dir, $login, $password, $secret);
	}
}